#!/usr/bin/env python

from pandas_ml.skaccessors.base import _maybe_sklearn_data
from pandas_ml.skaccessors.cluster import ClusterMethods
from pandas_ml.skaccessors.covariance import CovarianceMethods
from pandas_ml.skaccessors.cross_decomposition import CrossDecompositionMethods
from pandas_ml.skaccessors.cross_validation import CrossValidationMethods
from pandas_ml.skaccessors.decomposition import DecompositionMethods
from pandas_ml.skaccessors.ensemble import EnsembleMethods
from pandas_ml.skaccessors.feature_extraction import FeatureExtractionMethods
from pandas_ml.skaccessors.feature_selection import FeatureSelectionMethods
from pandas_ml.skaccessors.gaussian_process import GaussianProcessMethods
from pandas_ml.skaccessors.grid_search import GridSearchMethods
from pandas_ml.skaccessors.isotonic import IsotonicMethods
from pandas_ml.skaccessors.learning_curve import LearningCurveMethods
from pandas_ml.skaccessors.linear_model import LinearModelMethods
from pandas_ml.skaccessors.manifold import ManifoldMethods
from pandas_ml.skaccessors.metrics import MetricsMethods
from pandas_ml.skaccessors.neighbors import NeighborsMethods
from pandas_ml.skaccessors.pipeline import PipelineMethods
from pandas_ml.skaccessors.preprocessing import PreprocessingMethods
from pandas_ml.skaccessors.svm import SVMMethods
